// DGen/SDL 1.17
// by Joe Groff <joe@pknet.com>
// Read LICENSE for copyright etc., but if you've seen one BSDish license,
// you've seen them all ;)

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>


#include <fcntl.h>
#include <string.h>
#include <stdint.h>
#include <limits.h>
#include <errno.h>

#include <SDL/SDL.h>

#ifdef NSPIRE
	#include <nucleus.h>
#endif

#include "data.h"
#include "gui.h"

#define IS_MAIN_CPP
#include "system.h"
#include "md.h"
#include "pd.h"
#include "pd-defs.h"
#include "rc.h"
#include "rc-vars.h"

// Defined in ras.cpp, and set to true if the Genesis palette's changed.
extern int pal_dirty;

FILE *debug_log = NULL;

// Do a demo frame, if active
enum demo_status {
	DEMO_OFF,
	DEMO_RECORD,
	DEMO_PLAY
};

static inline void do_demo(md& megad, FILE* demo, enum demo_status* status)
{
	uint32_t pad[2];

	switch (*status) {
	case DEMO_OFF:
		break;
	case DEMO_RECORD:
		pad[0] = h2be32(megad.pad[0]);
		pad[1] = h2be32(megad.pad[1]);
		fwrite(&pad, sizeof(pad), 1, demo);
		break;
	case DEMO_PLAY:
		if (fread(&pad, sizeof(pad), 1, demo) == 1) {
			megad.pad[0] = be2h32(pad[0]);
			megad.pad[1] = be2h32(pad[1]);
		}
		else {
			if (feof(demo))
				pd_message("Demo finished.");
			else
				pd_message("Demo finished (read error).");
			*status = DEMO_OFF;
		}
		break;
	}
}

// Temporary garbage can string :)
static char temp[65536] = "";

// Show help and exit with code 2
static void help()
{
  printf(
  "DGen/SDL v"VER"\n"
  "Usage: dgen [options] [romname [...]]\n\n"
  "Where options are:\n"
  "    -v              Print version number and exit.\n"
  "    -r RCFILE       Read in the file RCFILE after parsing\n"
  "                    $HOME/.dgen/dgenrc.\n"
  "    -n USEC         Causes DGen to sleep USEC microseconds per frame, to\n"
  "                    be nice to other processes.\n"
  "    -p CODE,CODE... Takes a comma-delimited list of Game Genie (ABCD-EFGH)\n"
  "                    or Hex (123456:ABCD) codes to patch the ROM with.\n"
  "    -R (J|X|U|E| )  Force emulator region. Affects vertical resolution,\n"
  "                    frame rate and ROM operation.\n"
  "                    J: Japan (NTSC), X: Japan (PAL), U: America (NTSC)\n"
  "                    E: Europe (PAL), ' ': auto (default).\n"
  "    -N              Use NTSC mode (60Hz).\n"
  "    -P              Use PAL mode (50Hz).\n"
  "    -H HZ           Use a custom frame rate.\n"
  "    -d DEMONAME     Record a demo of the game you are playing.\n"
  "    -D DEMONAME     Play back a previously recorded demo.\n"
  "    -s SLOT         Load the saved state from the given slot at startup.\n"
#ifdef __MINGW32__
  "    -m              Do not detach from console.\n"
#endif
  );
  // Display platform-specific options
  pd_help();
  exit(2);
}

// Save/load states
// It is externed from your implementation to change the current slot
// (I know this is a hack :)
int slot = 0;
void md_save(md& megad)
{
	FILE *save;
	char file[64];

	if (!megad.plugged) {
		pd_message("Cannot save state when no ROM is loaded.");
		return;
	}
	if (((size_t)snprintf(file,
			      sizeof(file),
			      "%s.gs%d",
			      megad.romname,
			      slot) >= sizeof(file)) ||
	    ((save = dgen_fopen("saves", file, DGEN_WRITE)) == NULL)) {
		snprintf(temp, sizeof(temp),
			 "Couldn't save state to slot %d!", slot);
		pd_message(temp);
		return;
	}
	megad.export_gst(save);
	fclose(save);
	snprintf(temp, sizeof(temp), "Saved state to slot %d.", slot);
	pd_message(temp);
}

void md_load(md& megad)
{
	FILE *load;
	char file[64];

	if (!megad.plugged) {
		pd_message("Cannot restore state when no ROM is loaded.");
		return;
	}
	if (((size_t)snprintf(file,
			      sizeof(file),
			      "%s.gs%d",
			      megad.romname,
			      slot) >= sizeof(file)) ||
	    ((load = dgen_fopen("saves", file, DGEN_READ)) == NULL)) {
		snprintf(temp, sizeof(temp),
			 "Couldn't load state from slot %d!", slot);
		pd_message(temp);
		return;
	}
	megad.import_gst(load);
	fclose(load);
	snprintf(temp, sizeof(temp), "Loaded state from slot %d.", slot);
	pd_message(temp);
}

// Load/save states from file
void ram_save(md& megad)
{
	FILE *save;
	int ret;

	if (!megad.has_save_ram())
		return;
	save = dgen_fopen("ram", megad.romname, DGEN_WRITE);
	if (save == NULL)
		goto fail;
	ret = megad.put_save_ram(save);
	fclose(save);
	if (ret == 0)
		return;
fail:
	fprintf(stderr, "Couldn't save battery RAM to `%s'\n", megad.romname);
}

void ram_load(md& megad)
{
	FILE *load;
	int ret;

	if (!megad.has_save_ram())
		return;
	load = dgen_fopen("ram", megad.romname, DGEN_READ);
	if (load == NULL)
		goto fail;
	ret = megad.get_save_ram(load);
	fclose(load);
	if (ret == 0)
		return;
fail:
	fprintf(stderr, "Couldn't load battery RAM from `%s'\n",
		megad.romname);
}



int dgen(char* rom_path)
{
  int c = 0, stop = 0, usec = 0, start_slot = -1;
  unsigned long frames, frames_old, fps;
  char *patches = NULL, *rom = NULL;
  unsigned long oldclk, newclk, startclk, fpsclk;
  FILE *file = NULL;
  enum demo_status demo_status = DEMO_OFF;
  unsigned int samples;
  class md *megad;
  bool first = true;
  bool forced_hz = false;
  bool forced_pal = false;
  
	// Initialize the platform-dependent stuff.
	if (!pd_graphics_init(dgen_sound, dgen_pal, dgen_hz))
    {
      fprintf(stderr, "main: Couldn't initialize graphics!\n");
      return 1;
    }
	
	rom = rom_path;
	
	// Create the megadrive object.
	megad = new md(dgen_pal, dgen_region);
	if ((megad == NULL) || (!megad->okay())) {
		fprintf(stderr, "main: Mega Drive initialization failed.\n");
		goto clean_up;
	}
next_rom:
	// Load the requested ROM.
	if (rom != NULL) 
	{
		if (megad->load(rom)) {
			pd_message("Unable to load \"%s\".", rom);
			/*if ((first) && ((optind + 1) == argc))*/
				goto clean_up;
		}
		else
			pd_message("Loaded \"%s\".", rom);
	}
	else
	{
		pd_message("No cartridge.");
	}
	
	first = false;
	// Set untouched pads.
	megad->pad[0] = MD_PAD_UNTOUCHED;
	megad->pad[1] = MD_PAD_UNTOUCHED;
	
	// Reset
	megad->reset();

	// Automatic region settings from ROM header.
	if (!dgen_region) 
	{
		uint8_t c = megad->region_guess();
		int hz;
		int pal;

		md::region_info(c, &pal, &hz, 0, 0, 0);
		if (forced_hz)
			hz = dgen_hz;
		if (forced_pal)
			pal = dgen_pal;
		if ((hz != dgen_hz) || (pal != dgen_pal) ||
		    (c != megad->region)) {
			megad->region = c;
			dgen_hz = hz;
			dgen_pal = pal;
			printf("main: reconfiguring for region \"%c\": "
			       "%dHz (%s)\n", c, hz, (pal ? "PAL" : "NTSC"));
			pd_graphics_reinit(dgen_sound, dgen_pal, dgen_hz);
			if (dgen_sound) {
				long rate = dgen_soundrate;

				pd_sound_deinit();
				samples = (dgen_soundsegs * (rate / dgen_hz));
				pd_sound_init(rate, samples);
			}
			megad->pal = pal;
			megad->init_pal();
			megad->init_sound();
		}
	}

/*
	// Load up save RAM
	ram_load(*megad);
	// If -s option was given, load the requested slot
	if (start_slot >= 0) {
		slot = start_slot;
		md_load(*megad);
	}
	// If autoload is on, load save state 0
	else if (dgen_autoload) {
		slot = 0;
		md_load(*megad);
	}
*/
	// Start the timing refs
	startclk = pd_usecs();
	oldclk = startclk;
	fpsclk = startclk;

	// Show cartridge header
	if (dgen_show_carthead)
		pd_show_carthead(*megad);

	// Go around, and around, and around, and around... ;)
	frames = 0;
	frames_old = 0;
	fps = 0;
	while (!stop) 
	{
		const unsigned int usec_frame = (1000000 / dgen_hz);
		unsigned long tmp;
		int frames_todo;

		newclk = pd_usecs();

		if (pd_stopped()) {
			// Fix FPS count.
			tmp = (newclk - oldclk);
			startclk += tmp;
			fpsclk += tmp;
			oldclk = newclk;
		}

		// Update FPS count.
		tmp = ((newclk - fpsclk) & 0x3fffff);
		if (tmp >= 1000000) {
			fpsclk = newclk;
			if (frames_old > frames)
				fps = (frames_old - frames);
			else
				fps = (frames - frames_old);
			frames_old = frames;
		}

		if (dgen_frameskip == 0) 
		{
			// Check whether megad->one_frame() must be called.
			if (pd_freeze)
				goto frozen;
			goto do_not_skip;
		}

		// Measure how many frames to do this round.
		usec += ((newclk - oldclk) & 0x3fffff); // no more than 4 secs
		frames_todo = (usec / usec_frame);
		usec %= usec_frame;
		oldclk = newclk;

		if (frames_todo == 0) {
			// No frame to do yet, relax the CPU until next one.
			tmp = (usec_frame - usec);
			if (tmp > 1000) {
				// Never sleep for longer than the 50Hz value
				// so events are checked often enough.
#ifndef NSPIRE
				if (tmp > (1000000 / 50))
					tmp = (1000000 / 50);
				tmp -= 1000;
#endif
#ifdef __BEOS__
				snooze(tmp / 1000);
#else
	#ifndef NSPIRE
				usleep(tmp);
	#endif
#endif
			}
		}
		else {
			// Check whether megad->one_frame() must be called.
			if (pd_freeze)
				goto frozen;

			// Draw frames.
			while (frames_todo != 1) {
				do_demo(*megad, file, &demo_status);
				/*
				if (dgen_sound) 
				{
					// Skip this frame, keep sound going.
					megad->one_frame(NULL, NULL, &sndi);
					pd_sound_write();
				}
				else*/
					megad->one_frame(NULL, NULL, NULL);
				--frames_todo;
				stop |= (pd_handle_events(*megad) ^ 1);
			}
			--frames_todo;
		do_not_skip:
			do_demo(*megad, file, &demo_status);
			if (dgen_sound) {
				megad->one_frame(&mdscr, mdpal, &sndi);
				pd_sound_write();
			}
			else
				megad->one_frame(&mdscr, mdpal, NULL);
		frozen:
			if ((mdpal) && (pal_dirty)) {
				pd_graphics_palette_update();
				pal_dirty = 0;
			}
			pd_graphics_update(megad->plugged);
			++frames;
		}

		stop |= (pd_handle_events(*megad) ^ 1);

		if (dgen_nice) {
#ifdef __BEOS__
			snooze(dgen_nice);
#else
			//usleep(dgen_nice);
#endif
		}
	}


	// Print fps
	/*fpsclk = ((pd_usecs() - startclk) / 1000000);
	if (fpsclk == 0)
		fpsclk = 1;
		
	printf("%lu frames per second (average %lu, optimal %ld)\n",
	       fps, (frames / fpsclk), (long)dgen_hz);
*/
	/*ram_save(*megad);
	if (dgen_autosave) {
		slot = 0;
		md_save(*megad);
	}
	megad->unplug();
	if (file) {
		fclose(file);
		file = NULL;
	}
	if ((++optind) < argc) {
		rom = argv[optind];
		stop = 0;
		goto next_rom;
	}*/
	
clean_up:
	delete megad;
	pd_sound_deinit();
	pd_quit();
	return 0;
}



SDL_Surface *gui_screen;
SDL_Rect gui_dst, gui_dst2;
SDL_Event gui_event;

/* Setting this low enough crashes the program, why ?*/
#define MAX_LENGH 192

unsigned char esc, up_button, down_button, ctrl_button;

char file_name[MAX_LENGH][MAX_LENGH];
short file_type[MAX_LENGH];

unsigned char UP_state = 0, UP_time = 0;
unsigned char DOWN_state = 0, DOWN_time = 0;
unsigned char CTRL_state = 0, CTRL_time = 0;

char* currentdir;
char* last_folder;

/* This was done in order to save some memory on limited platforms*/
#if MAX_LENGH < 255
	unsigned char fileid_selected;
	unsigned char choice;
	unsigned char scroll_choice;
	unsigned char numb_files;
#elif MAX_LENGH < 65000
	unsigned short fileid_selected;
	unsigned short choice;
	unsigned short scroll_choice;
	unsigned short numb_files;
#else
	unsigned long fileid_selected;
	unsigned long choice;
	unsigned long scroll_choice;
	unsigned long numb_files;
#endif

#define BLUE_C  255
#define GREEN_C 512
#define TUR_C 750
#define F_C  1023

void init(void) 
{
	SDL_Init(SDL_INIT_VIDEO);
#ifdef NSPIRE
	gui_screen = SDL_SetVideoMode(320, 240, has_colors ? 16 : 8, SDL_SWSURFACE ); 
#else
	gui_screen = SDL_SetVideoMode(320, 240, 16, SDL_SWSURFACE ); 
#endif

	SDL_ShowCursor(SDL_DISABLE);
	SDL_WM_SetCaption("SDL File Browser", NULL);
	
	up_button = 0;
	down_button = 0;
	esc = 0;
	ctrl_button = 0;

    /* Set the size of the clearing area. x, w, h never change so they are set here */	
	gui_dst.x = 16;
	gui_dst.w = 16;
	gui_dst.h = 48;
	
    /* Used to define the whole screen to be cleared */	
	gui_dst2.x = 0;
	gui_dst2.y = 0;
	gui_dst2.w = 320;
	gui_dst2.h = 240;
	
	choice = 0;
	
	/* scroll_choice = 0 means that files from id 0 to 11 (file_name) will be shown on screen  */	
	scroll_choice = 0;
}


/*

	MY GUI HERE
	COPYRIGHT (C) 2015 , Bruno Marie
	GPLv2

*/


int main() 
{
	/* Buffer to hold the current directory */
	char* buf = NULL;
	unsigned char rom_chosen;
	unsigned char done = 1;
	rom_chosen = 0;
	
	/* Temporary array, used for starting executable*/
	char cacasd[256];
	
	/* Init video and variables */
	init();
	
	/* Set it to the current directory. */
	currentdir = getcwd(buf, MAX_LENGH);
	
	/* List 12 files to be shown on screen (here, it is the first chunck) */
	list_all_files(currentdir);
	
	/* Refresh everything on screen */
	refresh_cursor(1);
	
	
	while (!esc) 
	{
		while (esc==0)
		{
			/* Call function for input */
			controls();
			
				/* If Up button is pressed down... */
				if (UP_state == 1)
				{
					if (choice > 0) 
					{
						choice--;
						refresh_cursor(0);
						set_fileid();
					}
					else if (scroll_choice > 0)
					{
						choice = 11;
						scroll_choice = scroll_choice - 1;
						refresh_cursor(3);
						set_fileid();
					}
				}
				/* If Down button is pressed down... */
				else if (DOWN_state == 1)
				{
					/* Don't let the user to scroll more than there are files... */
					if (fileid_selected < numb_files)
					{
						if (choice < 11) 
						{
							choice++;	
							refresh_cursor(0);
							set_fileid();
						}
						/* If the user wants to go down and there are more files, change the files to shown to the next one (thanks to scroll_choice) */
						else if (numb_files > 10)
						{
							scroll_choice = scroll_choice + 1;
							choice = 0;
							set_fileid();
							refresh_cursor(3);
						}
					}
				}
			
			/* If Control button is pressed... */
			if (CTRL_state == 1)
			{
				/* If file is a tns file then launch it */
				if (file_type[fileid_selected] == BLUE_C) 
				{
					sprintf(cacasd, "%s/%s", currentdir, file_name[fileid_selected]);
					rom_chosen = 1;
					esc = 1;
				}
				/* If not then go to that folder */
				else
				{
					goto_folder();
				}
			}
			
			/* Don't waste CPU cycles */
			SDL_Delay(16);
		}
		
		printf("You picked your rom...\n");
		
		
		if (rom_chosen == 1)
		{
			if (gui_screen != NULL) SDL_FreeSurface(gui_screen);
			SDL_QuitSubSystem(SDL_INIT_VIDEO);
			
			printf("Going inside of LOOP\n");
			
			while(done == 1)
			{
				printf("Starting dgen thingy...\n");
				dgen(cacasd);
				done = 0;
			}
			
			done = 1;
			rom_chosen = 0;
			init();
			currentdir = getcwd(buf, MAX_LENGH);
			refresh_cursor(2);
		}
	
	}
	
	if (gui_screen != NULL) SDL_FreeSurface(gui_screen);
	SDL_QuitSubSystem(SDL_INIT_VIDEO);
	SDL_Quit();
	
	exit(0);
}

void controls()
{
    Uint8 *keystate = SDL_GetKeyState(NULL);
    
	if (UP_state == 0)
	{ 
		if (up_button)
		{
			UP_state = 1;
			UP_time = 0;
		}
	}
	else if (UP_state == 1) 
	{
		UP_time = UP_time + 1;
		
		if (UP_time > 0)
		{
			UP_state = 2;
			UP_time = 0;
		}
	}
	else if (UP_state == 2)
	{ 
		
		if (!(up_button))
		{
			UP_state = 3;
			UP_time = 0;
		}
	}
	else if (UP_state == 3) 
	{ 
		UP_time = UP_time + 1;
		
		if (UP_time > 1)
		{
			UP_state = 0;
			UP_time = 0;
		}
	}
    
	if (DOWN_state == 0)
	{ 
		if (down_button)
		{
			DOWN_state = 1;
			DOWN_time = 0;
		}
	}
	else if (DOWN_state == 1)  
	{
		DOWN_time = DOWN_time + 1;
		
		if (DOWN_time > 0)
		{
			DOWN_state = 2;
			DOWN_time = 0;
		}
	}
	else if (DOWN_state == 2)
	{ 
		
		if (!(down_button))
		{
			DOWN_state = 3;
			DOWN_time = 0;
		}
	}
	else if (DOWN_state == 3) 
	{ 
		DOWN_time = DOWN_time + 1;
		
		if (DOWN_time > 1)
		{
			DOWN_state = 0;
			DOWN_time = 0;
		}
	}
    
	if (CTRL_state == 0)
	{ 
		if (ctrl_button)
		{
			CTRL_state = 1;
			CTRL_time = 0;
		}
	}
	else if (CTRL_state == 1)  
	{
		CTRL_time = CTRL_time + 1;
		
		if (CTRL_time > 0)
		{
			CTRL_state = 2;
			CTRL_time = 0;
		}
	}
	else if (CTRL_state == 2)
	{ 
		
		if (!(ctrl_button))
		{
			CTRL_state = 3;
			CTRL_time = 0;
		}
	}
	else if (CTRL_state == 3) 
	{ 
		CTRL_time = CTRL_time + 1;
		
		if (CTRL_time > 1)
		{
			CTRL_state = 0;
			CTRL_time = 0;
		}
	}
    
	if (keystate[SDLK_UP])
	{
		up_button = 1;
	}
	else
	{
		up_button = 0;
	}
	
	if (keystate[SDLK_DOWN])
	{
		down_button = 1;
	}
	else
	{
		down_button = 0;
	}
	
	if (keystate[SDLK_LCTRL])
	{
		ctrl_button = 1;
	}
	else
	{
		ctrl_button = 0;
	}
    
	if (keystate[SDLK_ESCAPE])
	{
		esc = 1;
	}
	else
	{
		esc = 0;
	}
	
    SDL_Event event;
    SDL_PollEvent(&event);
}

void set_fileid()
{
	/* fileid_selected is the id of the file selected (first dimension of the file_name array) */
	fileid_selected = choice + (scroll_choice*12);
}

void goto_folder()
{
	sprintf(currentdir, "%s/%s", currentdir, file_name[choice]);
	
	#ifdef NSPIRE
		/* Set the current directory (with lots of "..") to the chosen one */
		NU_Set_Current_Dir(currentdir);
		/* This will return the current directory properly */
		NU_Current_Dir("A:",currentdir);
	#endif
	
	list_all_files(currentdir);
	refresh_cursor(2);
}

/*
	Draw the list of files on-screen (it is divided by the scroll_choice variable)
	For example, scroll_choice = 1 means that files from id 12 to 23 will be shown etc...
*/
void draw_files_list()
{
	unsigned char i;
	
	for (i = 0; i < 12; i++)
	{
		print_string(file_name[i+(scroll_choice*12)],file_type[i+(scroll_choice*12)],0,48,40+(16*i));
	}
}


void refresh_cursor(unsigned char all)
{	
	if (all == 2)
	{
		choice = 0;
		scroll_choice = 0;
		set_fileid();
		SDL_FillRect(gui_screen, &gui_dst2, 0);
	}
	else if (all == 3)
	{
		SDL_FillRect(gui_screen, &gui_dst2, 0);
	}
	
	/* Update position of cursor so we can limit the area to refresh*/
	gui_dst.y = 40+(16*choice) - 16;
	SDL_FillRect(gui_screen, &gui_dst, 0);
	
	/* Then draw the cursor again after being cleared, it will be shown after we passed SDL_Flip/UpdateRect to it*/
	print_string("=>",GREEN_C,0,16,40+(choice*16) );
	
	draw_files_list();
	
	if (all == 1 || all == 2 || all == 3)
	{
		print_string(currentdir,GREEN_C,0,8,16);
		print_string("SELECT A MEGADRIVE/GENEIS ROM", 1500, 0, 8, 6);
		SDL_Flip(gui_screen);
	}
	else
	{
		SDL_UpdateRect(gui_screen, gui_dst.x, gui_dst.y, gui_dst.w, gui_dst.h);
	}
}

void list_all_files(char* directory)
{
	DIR *dir;
	struct dirent *ent;
	
	unsigned char temp;
	
	#if MAX_LENGH < 255
		unsigned char i;
	#elif MAX_LENGH < 65000
		unsigned short i;
	#else
		unsigned long i;
	#endif

	short pc;
	
	/* Reset all the stored files to zero*/
	for(i=0;i<MAX_LENGH;i++)
	{
		strcpy(file_name[i], "");
	}
	
	i = 0;
	temp = 0;
	pc = -1;
	numb_files = 0;
	
	if ((dir = opendir (directory)) != NULL) 
	{
		while ( (ent = readdir (dir)) != NULL ) 
		{
			/* Add the .. string and then after, reject them all*/
			if (i == 0)
			{
				strcpy(file_name[i], "..");
				file_type[i] = TUR_C;
				i++;
				numb_files++;
			}
			
			/* Finds .tns occurence */
			char* pch = strstr (ent->d_name,"md.tns");
			
			/* Reject these two signs */
			char* pch2 = strstr (ent->d_name,"..");
			pc = strncmp (ent->d_name, ".", 2);
			
			char* pch4 = strstr (ent->d_name,"bin.tns");
			
			/* Check if file in question is a folder */
			temp = is_folder(ent->d_name);
			
			/* If file has ".tns" extension, is a folder AND is not ".." and "." */
			if ( ((pch != NULL || pch4 != NULL) || (temp)) && (pch2 == NULL && pc != 0) )
			{
				/* Copy string cotent from ent->d_name to file_name[i]*/
				strcpy(file_name[i], ent->d_name);
				
				if (pch != NULL || pch4 != NULL)
				{
					file_type[i] = BLUE_C;
				}
				else
				{
					file_type[i] = F_C;
				}

				i++;
				numb_files++;
			}
			
		}
		closedir (dir);
	} 
	else 
	{
		perror ("Not a directory");
	}
	
	numb_files = numb_files - 1;
}

/* Draw a char on-screen. Thanks Exphophase for the code ! (coming form Gpsp, GPLv2) */
void screen_showchar(int x, int y, unsigned char a, int fg_color, int bg_color) 
{
	unsigned short *dst;
	int w, h;

	SDL_LockSurface(gui_screen);
	
	for(h = 8; h; h--) 
	{
		dst = (unsigned short *)gui_screen->pixels + (y+8-h)*gui_screen->w + x;
		for(w = 8; w; w--) 
		{
			unsigned short color = *dst; // background
			if((fontdata8x8[a*8 + (8-h)] >> w) & 1) color = fg_color;
			*dst++ = color;
		}
	}
	
	SDL_UnlockSurface(gui_screen);
}

/* Draw a string on-screen. Thanks Exphophase for the code ! (coming form Gpsp, GPLv2) */
void print_string(char *s, unsigned short fg_color, unsigned short bg_color, int x, int y) 
{
	int i;
	int j;
	j = strlen(s);
	
	for(i = 0; i < j; i++, x += 6) 
	{
		screen_showchar(x, y, s[i], fg_color, bg_color);
	}
}

/* Is the string a folder ? */
unsigned char is_folder(char* str1)
{
	struct stat st;
	unsigned char temp;
	
	if( stat( str1, &st ) == 0 && S_ISDIR( st.st_mode ) ) 
	{
		temp = 1;
	} 
	else 
	{
		temp = 0;
	}
	 
	return temp;
}

